"""Asteroid Odyssey SDK for space exploration simulations."""

from asteroid_odyssey.client import AsteroidClient


# # import apis into sdk package
# from .api.generated.asteroid_agents_api_client.api.api_api import APIApi
# from .api.generated.asteroid_agents_api_client.api.agent_api import AgentApi
# from .api.generated.asteroid_agents_api_client.api.execution_api import ExecutionApi
# from .api.generated.asteroid_agents_api_client.api.optimiser_api import OptimiserApi
# from .api.generated.asteroid_agents_api_client.api.workflow_api import WorkflowApi
# from .api.generated.asteroid_agents_api_client.api.default_api import DefaultApi

# # import ApiClient
# from .api.generated.agents.asteroid_agents_api_client.api_response import ApiResponse
# from .api.generated.agents.asteroid_agents_api_client.api_client import ApiClient
# from .api.generated.agents.asteroid_agents_api_client.configuration import Configuration
# from .api.generated.agents.asteroid_agents_api_client.exceptions import OpenApiException
# from .api.generated.agents.asteroid_agents_api_client.exceptions import ApiTypeError
# from .api.generated.agents.asteroid_agents_api_client.exceptions import ApiValueError
# from .api.generated.agents.asteroid_agents_api_client.exceptions import ApiKeyError
# from .api.generated.agents.asteroid_agents_api_client.exceptions import ApiAttributeError
# from .api.generated.agents.asteroid_agents_api_client.exceptions import ApiException

# # import models into sdk package
from .api.generated.asteroid_agents_api_client.models.agent import Agent
from .api.generated.asteroid_agents_api_client.models.browser_session import BrowserSession
from .api.generated.asteroid_agents_api_client.models.create_workflow_request import CreateWorkflowRequest
from .api.generated.asteroid_agents_api_client.models.execution import Execution
from .api.generated.asteroid_agents_api_client.models.execution_status import ExecutionStatus
# from .api.generated.asteroid_agents_api_client.models.health_check200_response import HealthCheck200Response
# from .api.generated.asteroid_agents_api_client.models.health_check500_response import HealthCheck500Response

__version__ = "0.1.20"
__all__ = ["AsteroidClient"]



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.execution_dynamic_data import ExecutionDynamicData
    from ..models.execution_result import ExecutionResult
    from ..models.execution_status import ExecutionStatus


T = TypeVar("T", bound="Execution")


@_attrs_define
class Execution:
    """
    Attributes:
        id (UUID): Execution identifier.
        run_id (UUID): Run ID.
        workflow_id (UUID): Workflow ID.
        result (ExecutionResult): The result of the execution.
        created_at (datetime.datetime): The date and time the execution was created.
        dynamic_data (Union[Unset, ExecutionDynamicData]): Dynamic data to be merged into the saved workflow
            configuration. Example: {'name': 'Alice', 'model': 'gpt-4o'}.
        status (Union[Unset, ExecutionStatus]):
        error (Union[Unset, str]): The error that occurred during the execution.
    """

    id: UUID
    run_id: UUID
    workflow_id: UUID
    result: "ExecutionResult"
    created_at: datetime.datetime
    dynamic_data: Union[Unset, "ExecutionDynamicData"] = UNSET
    status: Union[Unset, "ExecutionStatus"] = UNSET
    error: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = str(self.id)

        run_id = str(self.run_id)

        workflow_id = str(self.workflow_id)

        result = self.result.to_dict()

        created_at = self.created_at.isoformat()

        dynamic_data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.dynamic_data, Unset):
            dynamic_data = self.dynamic_data.to_dict()

        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.to_dict()

        error = self.error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "run_id": run_id,
                "workflow_id": workflow_id,
                "result": result,
                "created_at": created_at,
            }
        )
        if dynamic_data is not UNSET:
            field_dict["dynamic_data"] = dynamic_data
        if status is not UNSET:
            field_dict["status"] = status
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.execution_dynamic_data import ExecutionDynamicData
        from ..models.execution_result import ExecutionResult
        from ..models.execution_status import ExecutionStatus

        d = src_dict.copy()
        id = UUID(d.pop("id"))

        run_id = UUID(d.pop("run_id"))

        workflow_id = UUID(d.pop("workflow_id"))

        result = ExecutionResult.from_dict(d.pop("result"))

        created_at = isoparse(d.pop("created_at"))

        _dynamic_data = d.pop("dynamic_data", UNSET)
        dynamic_data: Union[Unset, ExecutionDynamicData]
        if isinstance(_dynamic_data, Unset):
            dynamic_data = UNSET
        else:
            dynamic_data = ExecutionDynamicData.from_dict(_dynamic_data)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ExecutionStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ExecutionStatus.from_dict(_status)

        error = d.pop("error", UNSET)

        execution = cls(
            id=id,
            run_id=run_id,
            workflow_id=workflow_id,
            result=result,
            created_at=created_at,
            dynamic_data=dynamic_data,
            status=status,
            error=error,
        )

        execution.additional_properties = d
        return execution

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="BrowserSession")


@_attrs_define
class BrowserSession:
    """
    Attributes:
        id (Union[Unset, UUID]): Browser session identifier.
        browser_name (Union[Unset, str]): Browser name (Anchor, Browserbase, etc.)
        execution_id (Union[Unset, UUID]): Execution ID.
        cdp_url (Union[Unset, str]): CDP URL.
        debugger_url (Union[Unset, str]): Debugger URL.
        session_id (Union[Unset, str]): Session ID.
        session_url (Union[Unset, str]): Session URL.
        recording_url (Union[Unset, str]): Recording URL.
    """

    id: Union[Unset, UUID] = UNSET
    browser_name: Union[Unset, str] = UNSET
    execution_id: Union[Unset, UUID] = UNSET
    cdp_url: Union[Unset, str] = UNSET
    debugger_url: Union[Unset, str] = UNSET
    session_id: Union[Unset, str] = UNSET
    session_url: Union[Unset, str] = UNSET
    recording_url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        browser_name = self.browser_name

        execution_id: Union[Unset, str] = UNSET
        if not isinstance(self.execution_id, Unset):
            execution_id = str(self.execution_id)

        cdp_url = self.cdp_url

        debugger_url = self.debugger_url

        session_id = self.session_id

        session_url = self.session_url

        recording_url = self.recording_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if browser_name is not UNSET:
            field_dict["browser_name"] = browser_name
        if execution_id is not UNSET:
            field_dict["execution_id"] = execution_id
        if cdp_url is not UNSET:
            field_dict["cdp_url"] = cdp_url
        if debugger_url is not UNSET:
            field_dict["debugger_url"] = debugger_url
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if session_url is not UNSET:
            field_dict["session_url"] = session_url
        if recording_url is not UNSET:
            field_dict["recording_url"] = recording_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _id = d.pop("id", UNSET)
        id: Union[Unset, UUID]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = UUID(_id)

        browser_name = d.pop("browser_name", UNSET)

        _execution_id = d.pop("execution_id", UNSET)
        execution_id: Union[Unset, UUID]
        if isinstance(_execution_id, Unset):
            execution_id = UNSET
        else:
            execution_id = UUID(_execution_id)

        cdp_url = d.pop("cdp_url", UNSET)

        debugger_url = d.pop("debugger_url", UNSET)

        session_id = d.pop("session_id", UNSET)

        session_url = d.pop("session_url", UNSET)

        recording_url = d.pop("recording_url", UNSET)

        browser_session = cls(
            id=id,
            browser_name=browser_name,
            execution_id=execution_id,
            cdp_url=cdp_url,
            debugger_url=debugger_url,
            session_id=session_id,
            session_url=session_url,
            recording_url=recording_url,
        )

        browser_session.additional_properties = d
        return browser_session

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="File")


@_attrs_define
class File:
    """
    Attributes:
        id (UUID): Unique file identifier. Example: 1.
        execution_id (UUID): Execution ID associated with the file. Example: 123e4567-e89b-12d3-a456-426614174000.
        file_path (str): Full GCS object path (e.g., "2023-10-21_14-05-01/1697892305-screenshot.png"). Example:
            2023-10-21_14-05-01/1697892305-screenshot.png.
        file_ext (str): File extension. Example: png.
        file_name (str): File name. Example: 1697892305-screenshot.png.
        file_size (int): Size of the file in bytes. Example: 2048.
        mime_type (str): MIME type of the file. Example: image/png.
        created_at (datetime.datetime): Timestamp when the file record was created. Example: 2023-10-21T14:05:01Z.
        signed_url (str): Signed URL to download the file. Example: https://storage.googleapis.com/asteroid-
            files/123e4567-e89b-12d3-a456-426614174000/2023-10-21_14-05-01/1697892305-screenshot.png.
    """

    id: UUID
    execution_id: UUID
    file_path: str
    file_ext: str
    file_name: str
    file_size: int
    mime_type: str
    created_at: datetime.datetime
    signed_url: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = str(self.id)

        execution_id = str(self.execution_id)

        file_path = self.file_path

        file_ext = self.file_ext

        file_name = self.file_name

        file_size = self.file_size

        mime_type = self.mime_type

        created_at = self.created_at.isoformat()

        signed_url = self.signed_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "execution_id": execution_id,
                "file_path": file_path,
                "file_ext": file_ext,
                "file_name": file_name,
                "file_size": file_size,
                "mime_type": mime_type,
                "created_at": created_at,
                "signed_url": signed_url,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = UUID(d.pop("id"))

        execution_id = UUID(d.pop("execution_id"))

        file_path = d.pop("file_path")

        file_ext = d.pop("file_ext")

        file_name = d.pop("file_name")

        file_size = d.pop("file_size")

        mime_type = d.pop("mime_type")

        created_at = isoparse(d.pop("created_at"))

        signed_url = d.pop("signed_url")

        file = cls(
            id=id,
            execution_id=execution_id,
            file_path=file_path,
            file_ext=file_ext,
            file_name=file_name,
            file_size=file_size,
            mime_type=mime_type,
            created_at=created_at,
            signed_url=signed_url,
        )

        file.additional_properties = d
        return file

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_workflow_request_provider import CreateWorkflowRequestProvider
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_workflow_request_fields import CreateWorkflowRequestFields
    from ..models.result_schema import ResultSchema


T = TypeVar("T", bound="CreateWorkflowRequest")


@_attrs_define
class CreateWorkflowRequest:
    """
    Attributes:
        name (str): The name of the workflow. Example: My workflow.
        result_schema (ResultSchema): A JSON Schema that defines the expected structure and validation rules for
            workflow results. Example: {'$schema': 'https://json-schema.org/draft/2020-12/schema#', 'type': 'object',
            'required': ['name', 'age', 'skills'], 'properties': {'name': {'type': 'string', 'minLength': 2}, 'age':
            {'type': 'integer', 'minimum': 18, 'maximum': 120}, 'email': {'type': 'string', 'format': 'email'}, 'skills':
            {'type': 'array', 'items': {'type': 'string'}, 'minItems': 1}}}.
        fields (CreateWorkflowRequestFields): JSON object containing static workflow configuration (e.g. a
            prompt_template). Example: {'model': 'gpt-4o', 'version': '2024-02-01'}.
        prompts (List[str]): The prompts for the workflow. They can have variables in them. They will be merged with the
            dynamic data passed when the workflow is executed. Example: ['Your name is {{.name}}, you speak {{.language}}',
            'Your task is {{.task}}'].
        provider (CreateWorkflowRequestProvider): The Language Model Provider for the Workflow Example: openai.
        user_id (Union[Unset, UUID]): The ID of the user that this workflow belongs to. Example:
            123e4567-e89b-12d3-a456-426614174000.
    """

    name: str
    result_schema: "ResultSchema"
    fields: "CreateWorkflowRequestFields"
    prompts: List[str]
    provider: CreateWorkflowRequestProvider
    user_id: Union[Unset, UUID] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        result_schema = self.result_schema.to_dict()

        fields = self.fields.to_dict()

        prompts = self.prompts

        provider = self.provider.value

        user_id: Union[Unset, str] = UNSET
        if not isinstance(self.user_id, Unset):
            user_id = str(self.user_id)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "result_schema": result_schema,
                "fields": fields,
                "prompts": prompts,
                "provider": provider,
            }
        )
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_workflow_request_fields import CreateWorkflowRequestFields
        from ..models.result_schema import ResultSchema

        d = src_dict.copy()
        name = d.pop("name")

        result_schema = ResultSchema.from_dict(d.pop("result_schema"))

        fields = CreateWorkflowRequestFields.from_dict(d.pop("fields"))

        prompts = cast(List[str], d.pop("prompts"))

        provider = CreateWorkflowRequestProvider(d.pop("provider"))

        _user_id = d.pop("user_id", UNSET)
        user_id: Union[Unset, UUID]
        if isinstance(_user_id, Unset):
            user_id = UNSET
        else:
            user_id = UUID(_user_id)

        create_workflow_request = cls(
            name=name,
            result_schema=result_schema,
            fields=fields,
            prompts=prompts,
            provider=provider,
            user_id=user_id,
        )

        create_workflow_request.additional_properties = d
        return create_workflow_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.status import Status
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExecutionStatus")


@_attrs_define
class ExecutionStatus:
    """
    Attributes:
        execution_id (UUID): Execution ID.
        status (Status): Status of the execution.
        created_at (datetime.datetime): The date and time the execution status was created.
        reason (Union[Unset, str]): Reason for the status.
    """

    execution_id: UUID
    status: Status
    created_at: datetime.datetime
    reason: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        execution_id = str(self.execution_id)

        status = self.status.value

        created_at = self.created_at.isoformat()

        reason = self.reason

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "execution_id": execution_id,
                "status": status,
                "created_at": created_at,
            }
        )
        if reason is not UNSET:
            field_dict["reason"] = reason

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        execution_id = UUID(d.pop("execution_id"))

        status = Status(d.pop("status"))

        created_at = isoparse(d.pop("created_at"))

        reason = d.pop("reason", UNSET)

        execution_status = cls(
            execution_id=execution_id,
            status=status,
            created_at=created_at,
            reason=reason,
        )

        execution_status.additional_properties = d
        return execution_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Optional, Dict, Any, List, Union, Callable, Tuple
from uuid import UUID
import logging
import time
from enum import Enum
import os

from .api.generated.asteroid_agents_api_client.models import (
    CreateWorkflowRequest,
    WorkflowExecution,
    ExecutionStatus,
    Execution,
    WorkflowExecutionRequest,
    Agent,
    ResultSchema,
    CreateWorkflowRequestFields,
    CreateWorkflowRequestProvider
)
from .api.generated.asteroid_agents_api_client.client import Client as ApiClient
from .api.generated.asteroid_agents_api_client.api.execution.get_execution import sync_detailed as get_execution
from .api.generated.asteroid_agents_api_client.api.agent.get_agents import sync_detailed as asteroid_get_agents
from .api.generated.asteroid_agents_api_client.api.workflow.create_workflow import sync_detailed as create_workflow
from .api.generated.asteroid_agents_api_client.api.workflow.get_agent_workflow_executions import sync_detailed as get_agent_workflow_executions
from .api.generated.asteroid_agents_api_client.api.workflow.execute_workflow import sync_detailed as execute_workflow
logger = logging.getLogger(__name__)

class ExecutionTerminalState(Enum):
    """Terminal states for an execution"""
    COMPLETED = "completed"
    FAILED = "failed"
    CANCELLED = "cancelled"
    ERROR = "error"

agent_name = "iris"

class ExecutionResult:
    """Wrapper class for execution results"""
    def __init__(self, execution: Execution):
        self.execution_id = execution.id
        self.status = execution.status
        self.result = execution.result
        self.error = execution.error if hasattr(execution, 'error') else None
        self.created_at = execution.created_at
        self.completed_at = execution.completed_at if hasattr(execution, 'completed_at') else None

class AsteroidClient:
    """
    A high-level client for interacting with the Asteroid API.
    """
    def __init__(
        self,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        verify_ssl: bool = True
    ):
        """
        Initialize the Asteroid client.

        Args:
            api_key: API key for authentication. If not provided, will look for ASTEROID_API_KEY env var
            base_url: Base URL for the API. Defaults to production URL if not specified
            verify_ssl: Whether to verify SSL certificates. Defaults to True
        """
        self.api_key = api_key or os.getenv("ASTEROID_API_KEY")
        if not self.api_key:
            raise ValueError(
                "API key is required. Either pass it directly or set ASTEROID_API_KEY environment variable"
            )

        self.base_url = base_url or "https://odyssey.asteroid.ai/api/v1"
        # Initialize API client
        self.client = ApiClient(
            base_url=self.base_url,
            verify_ssl=verify_ssl,
            headers={"X-Asteroid-Agents-Api-Key": f"{self.api_key}"}
        )

    def get_agents(self) -> List["Agent"]:
        """
        Get list of available agents.
        
        Returns:
            List of agent details
        """
        try:
            result = asteroid_get_agents(client=self.client)
            if result.parsed is None:
                raise ValueError("No agents were returned from the API")
            if not isinstance(result.parsed, list):
                raise ValueError(f"The result is not of type list, it is of type: {type(result.parsed)}")
            return result.parsed
        except Exception as e:
            logger.error(f"Failed to get agents: {str(e)}")
            raise

    def create_workflow(
        self, 
        workflow_name: str,
        start_url: str,
        prompt: str,
        result_schema: Optional[ResultSchema] = None
    ) -> str:
        """
        Create a new workflow for an agent.

        Args:
            workflow_name: Name of the workflow
            start_url: Starting URL for the workflow
            prompt: Prompt for the workflow
            result_schema: Optional custom result schema. Currently not fully supported.

        Returns:
            Workflow ID

        Warning:
            Custom result schemas are not fully supported yet and will be added in a future update.
            Currently, only the default schema will be used regardless of input.
        """
        if result_schema is not None:
            logger.warning("Custom result schemas are not fully supported yet and will be ignored. Using default schema.")

        # Default result schema
        default_schema = ResultSchema()
        default_schema.additional_properties = {
            "properties": {
                "explanation": {
                    "description": "Detailed explanation of the result",
                    "type": "string"
                },
                "success": {
                    "description": "Whether the operation was successful",
                    "type": "boolean"
                }
            },
            "required": [
                "explanation",
                "success"
            ],
            "type": "object"
        }
        
        if result_schema is None:
            result_schema = default_schema

        try:
            fields = CreateWorkflowRequestFields()
            fields.additional_properties = {
                "workflow_name": workflow_name,
                "start_url": start_url
            }

            request = CreateWorkflowRequest(
                name=workflow_name,
                result_schema=result_schema,
                fields=fields,
                prompts=[prompt],
                provider=CreateWorkflowRequestProvider.OPENAI
            )

            result = create_workflow(
                agent_name=agent_name,
                body=request,
                client=self.client
            ).parsed
            if not isinstance(result, str):
                raise ValueError("The result is not of type str")
            return result
        except Exception as e:
            logger.error(f"Failed to create workflow: {str(e)}")
            raise

    def execute_workflow(
        self,
        workflow_id: UUID,
        execution_params: Dict[str, Any]
    ) -> str:
        """
        Execute an existing workflow.

        Args:
            workflow_id: ID of workflow to execute
            execution_params: Parameters for workflow execution

        Returns:
            Execution ID
        """
        try:
            # Convert execution_params to WorkflowExecutionRequest using from_dict
            request_body = WorkflowExecutionRequest.from_dict(execution_params)
            
            result = execute_workflow(
                workflow_id=workflow_id,
                body=request_body,
                client=self.client
            ).parsed
            if not isinstance(result, str):
                raise ValueError("The result is not of type str")
            return result
        except Exception as e:
            logger.error(f"Failed to execute workflow: {str(e)}")
            raise

    def get_workflow_executions(self) -> List[WorkflowExecution]:
        """
        Get list of workflow executions.

        Returns:
            List of workflow executions
        """
        try:
            result = get_agent_workflow_executions(
                agent_name=agent_name,
                client=self.client
            ).parsed
            if not isinstance(result, List):
                raise ValueError("The result is not of type List")
            return result
        except Exception as e:
            logger.error(f"Failed to get workflow executions: {str(e)}")
            raise

    def get_execution(self, execution_id: str) -> Execution:
        """
        Get the full execution details.

        Args:
            execution_id: ID of the execution to retrieve

        Returns:
            Execution object with full details
        """
        try:
            result = get_execution(id=UUID(execution_id), client=self.client).parsed
            if not isinstance(result, Execution):
                raise ValueError("The result is not of type Execution")
            return result
        except Exception as e:
            logger.error(f"Failed to get execution: {str(e)}")
            raise

    def get_execution_status(self, execution_id: str) -> ExecutionStatus:
        """
        Get the current status of an execution.

        Args:
            execution_id: ID of the execution to check

        Returns:
            Current execution status
        """
        execution = self.get_execution(execution_id)
        if not isinstance(execution.status, ExecutionStatus):
            raise ValueError("The execution status is not of type ExecutionStatus")
        return execution.status

    def get_execution_result(self, execution_id: str) -> ExecutionResult:
        """
        Get the result of an execution.

        Args:
            execution_id: ID of the execution to get results for

        Returns:
            ExecutionResult object containing status, result, and other metadata

        Raises:
            ValueError: If execution doesn't exist or hasn't completed
        """
        execution = self.get_execution(execution_id)
        return ExecutionResult(execution)

    def wait_for_execution(
        self,
        execution_id: str,
        polling_interval: float = 1.0,
        timeout: Optional[float] = None,
        status_callback: Optional[Callable[[ExecutionStatus], None]] = None
    ) -> ExecutionStatus:
        """
        Wait for an execution to reach a terminal state.

        Args:
            execution_id: ID of the execution to wait for
            polling_interval: Time in seconds between status checks
            timeout: Maximum time in seconds to wait. None means wait indefinitely
            status_callback: Optional callback function that will be called with each status update

        Returns:
            Final execution status

        Raises:
            TimeoutError: If timeout is reached before execution reaches terminal state
            ValueError: If execution_id is invalid
        """
        start_time = time.time()
        last_status = None

        while True:
            # Check if we've exceeded timeout
            if timeout and (time.time() - start_time) > timeout:
                raise TimeoutError(f"Execution {execution_id} did not complete within {timeout} seconds")

            # Get current status
            current_status = self.get_execution_status(execution_id)

            # Call status callback if status has changed
            if status_callback and current_status != last_status:
                status_callback(current_status)
            last_status = current_status

            # Check if we've reached a terminal state
            if current_status.status.value in [state.value for state in ExecutionTerminalState]:
                return current_status

            # Wait before next check
            time.sleep(polling_interval)

    def wait_for_execution_result(
        self,
        execution_id: str,
        polling_interval: float = 1.0,
        timeout: Optional[float] = None,
        status_callback: Optional[Callable[[ExecutionStatus], None]] = None
    ) -> ExecutionResult:
        """
        Wait for an execution to complete and get its result.

        Args:
            execution_id: ID of the execution to wait for
            polling_interval: Time in seconds between status checks
            timeout: Maximum time in seconds to wait. None means wait indefinitely
            status_callback: Optional callback function that will be called with each status update

        Returns:
            ExecutionResult object containing final status, result, and other metadata

        Raises:
            TimeoutError: If timeout is reached before execution completes
            ValueError: If execution_id is invalid
        """
        # Wait for execution to reach terminal state
        final_status = self.wait_for_execution(
            execution_id=execution_id,
            polling_interval=polling_interval,
            timeout=timeout,
            status_callback=status_callback
        )

        # Get the final result
        result = self.get_execution_result(execution_id)

        # If execution failed, include error information in logs
        if final_status in [ExecutionTerminalState.FAILED, ExecutionTerminalState.ERROR]:
            logger.error(f"Execution {execution_id} failed with error: {result.error}")

        return result

    def execute_workflow_and_get_result(
        self,
        workflow_id: UUID,
        execution_params: Dict[str, Any],
        polling_interval: float = 1.0,
        timeout: Optional[float] = None,
        status_callback: Optional[Callable[[ExecutionStatus], None]] = None
    ) -> ExecutionResult:
        """
        Execute a workflow and wait for its result.

        Args:
            workflow_id: ID of workflow to execute
            execution_params: Parameters for workflow execution
            polling_interval: Time in seconds between status checks
            timeout: Maximum time in seconds to wait. None means wait indefinitely
            status_callback: Optional callback function that will be called with each status update

        Returns:
            ExecutionResult object containing final status, result, and other metadata
        """
        # Start execution
        execution_id = self.execute_workflow(workflow_id, execution_params)

        # Wait for result
        return self.wait_for_execution_result(
            execution_id=execution_id,
            polling_interval=polling_interval,
            timeout=timeout,
            status_callback=status_callback
        )

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.credential import Credential


T = TypeVar("T", bound="CredentialsResponse")


@_attrs_define
class CredentialsResponse:
    """
    Attributes:
        workflow_name (str): The name of the workflow
        credentials (List['Credential']):
    """

    workflow_name: str
    credentials: List["Credential"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        workflow_name = self.workflow_name

        credentials = []
        for credentials_item_data in self.credentials:
            credentials_item = credentials_item_data.to_dict()
            credentials.append(credentials_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "workflow_name": workflow_name,
                "credentials": credentials,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.credential import Credential

        d = src_dict.copy()
        workflow_name = d.pop("workflow_name")

        credentials = []
        _credentials = d.pop("credentials")
        for credentials_item_data in _credentials:
            credentials_item = Credential.from_dict(credentials_item_data)

            credentials.append(credentials_item)

        credentials_response = cls(
            workflow_name=workflow_name,
            credentials=credentials,
        )

        credentials_response.additional_properties = d
        return credentials_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .agent import Agent
from .browser_session import BrowserSession
from .create_workflow_request import CreateWorkflowRequest
from .create_workflow_request_fields import CreateWorkflowRequestFields
from .create_workflow_request_provider import CreateWorkflowRequestProvider
from .credential import Credential
from .credentials_request import CredentialsRequest
from .credentials_response import CredentialsResponse
from .delete_execution_response_200 import DeleteExecutionResponse200
from .delete_execution_response_404 import DeleteExecutionResponse404
from .delete_workflow_response_200 import DeleteWorkflowResponse200
from .delete_workflow_response_404 import DeleteWorkflowResponse404
from .execution import Execution
from .execution_dynamic_data import ExecutionDynamicData
from .execution_result import ExecutionResult
from .execution_status import ExecutionStatus
from .file import File
from .health_check_response_200 import HealthCheckResponse200
from .health_check_response_500 import HealthCheckResponse500
from .optimisation_request import OptimisationRequest
from .progress_update import ProgressUpdate
from .result_schema import ResultSchema
from .slack_channel_request import SlackChannelRequest
from .status import Status
from .workflow import Workflow
from .workflow_execution import WorkflowExecution
from .workflow_execution_request import WorkflowExecutionRequest
from .workflow_fields import WorkflowFields

__all__ = (
    "Agent",
    "BrowserSession",
    "CreateWorkflowRequest",
    "CreateWorkflowRequestFields",
    "CreateWorkflowRequestProvider",
    "Credential",
    "CredentialsRequest",
    "CredentialsResponse",
    "DeleteExecutionResponse200",
    "DeleteExecutionResponse404",
    "DeleteWorkflowResponse200",
    "DeleteWorkflowResponse404",
    "Execution",
    "ExecutionDynamicData",
    "ExecutionResult",
    "ExecutionStatus",
    "File",
    "HealthCheckResponse200",
    "HealthCheckResponse500",
    "OptimisationRequest",
    "ProgressUpdate",
    "ResultSchema",
    "SlackChannelRequest",
    "Status",
    "Workflow",
    "WorkflowExecution",
    "WorkflowExecutionRequest",
    "WorkflowFields",
)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.result_schema import ResultSchema
    from ..models.workflow_fields import WorkflowFields


T = TypeVar("T", bound="Workflow")


@_attrs_define
class Workflow:
    """
    Attributes:
        id (UUID): Workflow identifier.
        user_id (UUID): The ID of the user who created the workflow. Example: 123e4567-e89b-12d3-a456-426614174000.
        result_schema (ResultSchema): A JSON Schema that defines the expected structure and validation rules for
            workflow results. Example: {'$schema': 'https://json-schema.org/draft/2020-12/schema#', 'type': 'object',
            'required': ['name', 'age', 'skills'], 'properties': {'name': {'type': 'string', 'minLength': 2}, 'age':
            {'type': 'integer', 'minimum': 18, 'maximum': 120}, 'email': {'type': 'string', 'format': 'email'}, 'skills':
            {'type': 'array', 'items': {'type': 'string'}, 'minItems': 1}}}.
        agent_id (UUID): Identifier of the associated agent.
        name (str): Workflow name.
        fields (WorkflowFields): Workflow configuration. Example: {'model': 'gpt-4o', 'version': '2024-02-01'}.
        prompts (List[str]): The prompts for the workflow. They can have variables in them. They will be merged with the
            dynamic data passed when the workflow is executed. Example: ['Your name is {{.name}}, you speak {{.language}}',
            'Your task is {{.task}}'].
        created_at (Union[Unset, datetime.datetime]): The date and time the workflow was created.
        prompt_variables (Union[Unset, List[str]]): The variables in the prompts. Example: ['name', 'language', 'task'].
    """

    id: UUID
    user_id: UUID
    result_schema: "ResultSchema"
    agent_id: UUID
    name: str
    fields: "WorkflowFields"
    prompts: List[str]
    created_at: Union[Unset, datetime.datetime] = UNSET
    prompt_variables: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = str(self.id)

        user_id = str(self.user_id)

        result_schema = self.result_schema.to_dict()

        agent_id = str(self.agent_id)

        name = self.name

        fields = self.fields.to_dict()

        prompts = self.prompts

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        prompt_variables: Union[Unset, List[str]] = UNSET
        if not isinstance(self.prompt_variables, Unset):
            prompt_variables = self.prompt_variables

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "user_id": user_id,
                "result_schema": result_schema,
                "agent_id": agent_id,
                "name": name,
                "fields": fields,
                "prompts": prompts,
            }
        )
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if prompt_variables is not UNSET:
            field_dict["prompt_variables"] = prompt_variables

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.result_schema import ResultSchema
        from ..models.workflow_fields import WorkflowFields

        d = src_dict.copy()
        id = UUID(d.pop("id"))

        user_id = UUID(d.pop("user_id"))

        result_schema = ResultSchema.from_dict(d.pop("result_schema"))

        agent_id = UUID(d.pop("agent_id"))

        name = d.pop("name")

        fields = WorkflowFields.from_dict(d.pop("fields"))

        prompts = cast(List[str], d.pop("prompts"))

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        prompt_variables = cast(List[str], d.pop("prompt_variables", UNSET))

        workflow = cls(
            id=id,
            user_id=user_id,
            result_schema=result_schema,
            agent_id=agent_id,
            name=name,
            fields=fields,
            prompts=prompts,
            created_at=created_at,
            prompt_variables=prompt_variables,
        )

        workflow.additional_properties = d
        return workflow

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ResultSchema")


@_attrs_define
class ResultSchema:
    """A JSON Schema that defines the expected structure and validation rules for workflow results.

    Example:
        {'$schema': 'https://json-schema.org/draft/2020-12/schema#', 'type': 'object', 'required': ['name', 'age',
            'skills'], 'properties': {'name': {'type': 'string', 'minLength': 2}, 'age': {'type': 'integer', 'minimum': 18,
            'maximum': 120}, 'email': {'type': 'string', 'format': 'email'}, 'skills': {'type': 'array', 'items': {'type':
            'string'}, 'minItems': 1}}}

    """

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        result_schema = cls()

        result_schema.additional_properties = d
        return result_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
